@@ -401,6 +401,42 @@ class Key(object):
 
 			
 		
+		#
+	def edit_comment(self, path=None, passphrase=None, comment=None):
+
+		# checks.
+		success, response = utils.__check_parameter__(old, "old", None)
+		if not success: return response
+		success, response = utils.__check_parameter__(passphrase, "passphrase", None)
+		if not success: return response
+		success, response = utils.__check_parameter__(comment, "comment", None)
+		if not success: return response
+		
+		# do.
+		output = utils.__execute__(["ssh-keygen", "-c", "-P", passphrase, "-C", comment, "-f", path])
+
+		# check fails.
+		if "incorrect passphrase supplied" in output:
+			response["error"] = f"Provided an incorrect passphrase for key [{path}]."
+			return response
+		elif "No such file or directory" in output:
+			response["error"] = f"Key [{path}] does not exist."
+			return response
+		
+		# check success.	
+		elif "Comment '" in output and "' applied" in output:
+			response["success"] = True
+			response["message"] = f"Successfully edited the comment of key [{path}]."
+			return response
+
+		# unknown.
+		else:
+			l = f"Failed to edit the comment of key [{path}]"
+			response["error"] = (f"{l}, error: "+output.replace("\n", ". ").replace(". .", ".")+".)").replace(". .",".").replace("\r","").replace("..",".")
+			return response
+
+			
+		
 		#
 	def generate(self, directory=None, passphrase=None, comment=""):
 
@@ -428,10 +464,10 @@ class Key(object):
 		if comment == None: comment = ""
 		if "[#id:" not in comment: comment += f" [#id:{utils.__generate_shell_string__(characters=48, numerical_characters=True, special_characters=True)}]"
 		comment = '-C "{}"'.format(comment)
-		passphrase = '-N '+passphrase#utils.__string_to_bash__(passphrase)
+		passphrase = f'-N "{passphrase}"'#utils.__string_to_bash__(passphrase)
 
 		# execute.
-		os.system(f'ssh-keygen -q -v -o -a 100 -t ed25519 "{passphrase}" {identity_file} {comment}')
+		os.system(f'ssh-keygen -q -v -o -a 100 -t ed25519 {passphrase} {identity_file} {comment}')
 		if not os.path.exists(private_key): 
 			utils.__delete_file_path__(directory, forced=True, sudo=True)
 			response["error"] = f"Failed to generate key [{directory}]."
@@ -846,6 +882,25 @@ class Agent(object):
 			self.SSH_AGENT_PID = output.split("SSH_AGENT_PID=")[1].split(";")[0]
 			os.environ["SSH_AUTH_SOCK"] = self.SSH_AUTH_SOCK
 			os.environ["SSH_AGENT_PID"] = self.SSH_AGENT_PID
+
+		# set agent nr.2.
+		try:
+			output = utils.__execute__([f"ssh-add", "-D"])
+		except: a=1
+		try:
+			output = utils.__execute__([f"ssh-add", "-k"])
+		except: a=1
+		try:
+			output = utils.__execute__(f"ssh-agent")
+			try: 
+				SSH_AUTH_SOCK = output.split("SSH_AUTH_SOCK=")[1].split(";")[0]
+				os.environ["SSH_AUTH_SOCK"] = SSH_AUTH_SOCK
+			except: a=1
+			try: 
+				SSH_AGENT_PID = output.split("SSH_AGENT_PID=")[1].split(";")[0]
+				os.environ["SSH_AGENT_PID"] = SSH_AGENT_PID
+			except: a=1
+		except: a=1
 		
 		#
 	def add(self, 
@@ -1070,7 +1125,7 @@ class Agent(object):
 		response = utils.__default_response__()
 
 		# list keys.
-		output = utils.__execute__(command=["ssh-add", "-l"], return_format="array")
+		output = utils.__execute__(command=["ssh-add", "-L"], return_format="array")
 		if "Failed to communicate" in output:
 			response["error"] = "Failed to communicate with the ssh-agent."
 			return response
@@ -1103,6 +1158,7 @@ class Agent(object):
 			return response
 
 		# retrieve id from public key.
+		"""
 		try:
 			public_key_id = public_key.split("[#id:")[1].split("]")[0]
 		except IndexError:
@@ -1120,6 +1176,16 @@ class Agent(object):
 					success = True 
 					break
 			except IndexError: a=1
+		"""
+
+		# list.
+		response = self.list()
+		if response["error"] != None: return response
+		success = False
+		for key in response["keys"]:
+			if public_key.replace("\n","") in key:
+				success = True 
+				break
 
 		# success.
 		response = utils.__default_response__()		
